% File:      plot_TF_gram.m
% Copyright (c) 2019 by S. Lawrence Marple Jr.

% This script produces either a 3-D waterfall perspective figure or a 2-D color image
% display figure of the nonstationary time-vs-frequency
% evolution of the power spectral density (PSD), stored in two-dimensional (2-D) array
% PSD [the 'gram'] assumed to already be created in the Matlab workspace memory. For
% example, any of the demonstration Matlab scripts SPECTRUM_DEMO_xxx can create such
% 2-D PSD grams.  The dimensions of PSD are [time,frequency]. A one-sided (positive
% frequencies only) PSD plot is created along the frequency axis if the signal being
% analyzed is real-valued, or a two-sided (both positive and negative frequencies) PSD
% plot is created along the frequency axis if the signal being analyzed is complex-valued
% data. The user has the option of labeling the frequency axis either in units of fraction-
% of-sampling-frequency (actual frequency divided by sampling frequency, so dimensionless
% units are 0 to 0.5 for real-valued signals and -0.5 to 0.5 for complex-valued signals),
% or in actual units of Hertz (given the sampling frequency). Time evolves from t=0 at the
% bottom left of the figure to t=tmax at the top left of the figure.

[num_times,num_freqs] = size(psd);
disp([' The analysis parameters yielded ',int2str(num_times),' gram lines to plot.']);
titletext = input(' Enter text for time-frequency gram title (no quotes around text): ','s');
plot_type = input(' Enter 3 [3-D waterfall] or 2 [2-D color image] to display time-frequency gram: ');

%****************************************************************************************************
if plot_type == 3
    
   disp(' Time vs frequency axes scaling choices:')
   disp('      0 --  samples vs fraction of sampling frequency [dimensionless frequency axis]')
   disp('      1 --  seconds (s) vs  Hertz (Hz) [suggest for test1987, radar, heart data')
   disp('      2 --  milliseconds vs KHz [suggest for bat and helicopter data]')
   axis_choice = input(' Enter axes scaling choice: ');
   if axis_choice == 0
       samp_time = 1;
       samp_freq = 1;
       xlabeltitle = 'Fraction of Sampling Freqeuncy';
       ylabeltitle = 'Center Sample Number';
   elseif axis_choice == 1
       samp_time = T;
       samp_freq = 1/T;
       xlabeltitle = 'Frequency (Hz)';
       ylabeltitle = 'Center Time (s)';
   elseif axis_choice == 2
       samp_time = 1000*T;
       samp_freq = 1/(1000*T);
       xlabeltitle = 'Frequency (KHz)';
       ylabeltitle = 'Center Time (ms)';
   end
   step_time = samp_time*(interval-int_overlap);
   step_freq = samp_freq/num_freqs;
   if any(any(imag(x)~=0))
       disp(' Data is complex-valued.')
       psd_save = psd;
       min_freq_index = 1;
   else
       disp(' Data is real-valued.')
       psd_save = psd(:,num_freqs/2+1:num_freqs);
       min_freq_index = num_freqs/2 + 1;
   end
   disp(' PSD gram of time-frequency analysis will be scaled in logarithmic units (dB).')
   dB_range = input(' Logarithmic range in dB that PSD is to be plotted (typical range is 40 to 80 dB): ');
   dB_psd = 10*log10( (1/max(max(psd_save)))*abs(psd_save) + 10^(-dB_range/10) );
   % Code above prevents taking logarithm of 0 or - numbers, while setting minimum dB plot level
   clear psd_save
   
   colormap(zeros(64,3));
   waterfall(((min_freq_index:num_freqs)-num_freqs/2-1)*step_freq,(1:num_times)*step_time,dB_psd)

   xlabel(xlabeltitle)
   ylabel(ylabeltitle)
   zlabel('Relative PSD in dB')
   title(['WATERFALL :  Localized Time vs Localized Frequency of ',titletext])

%*****************************************************************************************************
elseif plot_type == 2

   disp(' PSD gram of time-frequency analysis will be scaled and plotted in logarithmic units (dB).')
   disp(' Largest PSD value will be reference at 0 dB. Smaller PSD values will be negative dB.')
   disp(' dB top and dB bottom will now be specified. dB values above dB_top will be set to dB_top.')
   disp(' dB values below dB_bottom will be set to dB_bottom.')
   dB_top = input(' Enter dB_top choice: ');
   dB_bottom = input(' Enter dB_bottom choice: ');

   plot_spectral_gram(psd,x,1/T,interval,int_overlap,dB_top,dB_bottom,titletext)
   
end
%