% Script:     plot_correlation.m
% Copyright (c) 2019 by S. Lawrence Marple Jr.

% This script computes and plots the complex correlogram (autocorrelation
% sequence estimate) of either the test1987 data set or the radar data set.

disp(' Data source choices:  1--test1987, 2--doppler_radar')
source = input('Enter number of data source choice: ');
if source == 1
    load('test1987.dat')
    data = test1987(:,1) + 1i*test1987(:,2);
elseif source == 2
    load('doppler_radar.dat')
    data = doppler_radar(:,1) + 1i*doppler_radar(:,2);
else
    error('Number of data source must be 1 or 2.')
end
max_lag = 63;
method = input('Enter 1 for biased or 2 for unbiased autocorrelation estimates: ');
if method == 2
    r = correlation_sequence(max_lag,'unbiased',data);
else
    r = correlation_sequence(max_lag,'biased',data);
end

figure
t = -max_lag:max_lag;
subplot(211)
hold off
axis([-max_lag max_lag -2 2])
hold on
plot(t,real(r))
box on
grid off
xlabel('Lag  Time  Index')
ylabel('ACS  Amplitude')
title('Real  Part  of  Autocorrelation  Sequence  Estimate')
subplot(212)
hold off
axis([-max_lag max_lag -2 2])
hold on
plot(t,imag(r))
box on
grid off
xlabel('Lag  Time  Index')
ylabel('ACS  Amplitude')
title('Imaginary  Part  of  Autocorrelation  Sequence  Estimate')
%