% File:      plot_psd.m
% Copyright (c) 2019 by S. Lawrence Marple Jr.

% This script plots the power spectral density values (the 'spectrum').
% A one-sided (positive frequencies only) spectrum is plotted if a symmetric
% spectrum is detected (always the case for real-valued data), or two-sided
% (both positive and negative frequencies) if a nonsymmetric spectrum is
% detected (usually the case for complex-valued data). The user has the option
% of labeling the frequency axis in units of fraction-of-sampling-frequency or
% in units of Hertz.  To change dB plotting range, modify value of 'db_range'.

clf
dB_range = input(' Range in dB to be plotted (suggest initially 60 dB): ');
%subplot(211)        % This creates a half page plot, rather than full page

disp(' Frequency axis scaling choices:')
disp('      actual   --  frequency expressed in Hertz')
disp('      fraction --  frequency expressed as fraction of sampling frequency')
scale_type = input(' Enter string for scaling choice: ','s');
if strcmp(scale_type,'actual')
    samp_freq = 1/T;
    xlabel('Frequency  (Hertz)')
else
    samp_freq = 1;
    xlabel('Fraction of Sampling Frequency')
end
ylabel('Relative PSD  (dB)')
max_freq = .5*samp_freq;

% determine if spectrum is symmetric (always the case for real-valued data);
% only the positive half is plotted if the spectrum is symmetric
num_psd = length(psd);
diff = (psd(2:num_psd/2) - psd(num_psd:-1:num_psd/2+2))./psd(2:num_psd/2);
if all( abs(diff) < 1.e-6 )       % spectrum symmetric
    min_freq = 0;
    min_index = num_psd/2 + 1;
else                              % spectrum nonsymmetric
    min_freq = -.5*samp_freq;
    min_index = 1;
end

% Following line protects against taking log of zero or negative numbers
dB_psd = 10*log10((1/max(psd))*abs(psd) + 1.e-10);

f = -.5:1/num_psd:(.5-1/num_psd);
hold off
axis([min_freq max_freq -dB_range 5])
hold on
plot(samp_freq*f(min_index:num_psd),dB_psd(min_index:num_psd))
title('ESTIMATED SPECTRUM')
box on
grid on
%