% Script:     plot_psd_MC
% Copyright (c) 2019 by S. Lawrence Marple Jr.

% This script graphs the autospectra, magnitude squared coherence (MSC), and
% coherence phase estimates of a two-channel process. It assumes that the 2 x 2
% spectral density matrix over a spectrum of 'num_psd' frequency points has
% already been reconstituted into the four vector variables 'psd_ch1' (auto-
% spectrum of channel 1), 'psd_ch2' (autospectrum of channel 2), 'msc' (the
% magnitude-squared coherence), and 'coh_phas' (phase of the coherence)

num_psd = length(psd_ch1);
if any(any(imag(x(:,1))~=0))
    data_type = 1;
    disp(' Data is complex-valued.')
    f = (-.5:1/num_psd:(.5-1/num_psd))*Fs;
    freqs = (1:num_psd);
else
    data_type = 0;
    disp(' Data is real-valued.')
    f = (0:1/num_psd:(.5-1/num_psd))*Fs;
    freqs = (num_psd/2+1:num_psd);
end

clf
subplot(2,2,1)

if data_type == 1
    axis([-Fs/2 Fs/2 -60 5])
else
    axis([0 Fs/2 -60 5])
end
set(gca,'Box','on')
hold on
plot(f,10*log10(abs(psd_ch1(freqs))/max(psd_ch1(freqs)) + 1.e-10))
% normalization for AR(1) data source case is 1500
title('(a) Autospectrum of Channel 1')
xlabel('Frequency (Hz)')
ylabel('Power Spectral Density (dB)')

subplot(2,2,2)
if data_type == 1
    axis([-Fs/2 Fs/2 -60 5])
else
    axis([0 Fs/2 -60 5])
end
set(gca,'Box','on')
hold on
plot(f,10*log10(abs(psd_ch2(freqs))/max(psd_ch2(freqs)) + 1.e-10))
%normalization for AR(1) data source case is 1500
title('(b) Autospectrum of Channel 2')
xlabel('Frequency (Hz)')
ylabel('Power Spectral Density (dB)')

subplot(2,2,3)
if data_type == 1
    axis([-Fs/2 Fs/2 0 1.1])
else
    axis([0 Fs/2 0 1.1])
end
set(gca,'Box','on')
hold on
plot(f,msc(freqs))
title('(c) Magnitude Squared Coherence')
xlabel('Frequency (Hz)')
ylabel('MSC')

subplot(2,2,4)
if data_type == 1
    axis([-Fs/2 Fs/2 -180 180])
else
    axis([0 Fs/2 -180 180])
end
set(gca,'Box','on')
plot(f,(coh_phase(freqs)/pi)*180)
title('(d) Phase of Coherence')
xlabel('Frequency (Hz)')
ylabel('Degrees')

disp(' Press any key to replace coherence phase plot with time delay estimate plot.')
pause

% replace MSC Phase plot with Time Delay estimate plot
subplot(2,2,4,'replace')
if data_type== 1
    f(num_psd/2+1) = 1e6;
else
    f(1) = 1e6;
end
td = coh_phase(freqs)./(2*pi*abs(f)');
max_td = max(abs(td));
if data_type == 1
    axis([-Fs/2 Fs/2 -max_td max_td])
    f(num_psd/2+1) = 0;
else
    axis([0 Fs/2 -max_td max_td])
    f(1) = 0;
end
set(gca,'Box','on')
hold on
plot(f,td)
title('(d) Time Delay Vs Frequency')
xlabel(' Frequency (Hz)')
ylabel( 'Time Delay (sec)')
%