function plot_spectral_gramxx(psd,data,Fs,interval,int_overlap,dB_top,dB_bottom,titletext)
% Copyright (c) 2019 by S. Lawrence Marple Jr.

% psd      --  a two-dimensional array of PSD values created by "spectrum_demo_xxx.m", arranged
%              by columns of frequency bins vs rows of the center analysis time values; each row
%              contains all the frequency bins corresponding to a center time. Thus a row index
%              corresponds to particular center time and a column index corresponds to a particular
%              frequency bin in Hz (or cycles/time_unit if the time_unit is other than seconds)
% data     --  the original data array (complex-valued or real-valued) from which
%              the 2-D time-vs-frequency PSD gram values were generated
% Fs       --  the sampling rate in samples per second (Hz), or samples per
%              time unit if other than seconds
% interval --  the analysis interval used to create gram, in number of samples (use same
%              as entered in the "spectrum_demo_xxx.m" script
% int_overlap  --  the overlap between analysis center times, in number of samples (use same value
%              value as entered in the "spectrum_demo_xx.m" script
% dB_top    -- top dB value to color code the gram
% dB_bottom -- bottom dB value to color code the gram
% titletext -- text of gram plot title entered between quotes, such as 'Results for XXXX Data'

Ts=1/Fs;    % sampling interval (seconds or inverse units of sampling frequency if not seconds)
dB_range = -dB_bottom;
[num_times,num_freqs] = size(psd);
step = interval - int_overlap;
disp('Time vs frequency axes scaling choices:')
disp('      0 --  samples vs fraction of sampling frequency [dimensionless frequency axis]')
disp('      1 --  seconds (s) vs  Hertz (Hz) [suggest for test1987, doppler_radar, and heart data]')
disp('      2 --  milliseconds (ms) vs KiloHertz (KHz) [suggest for bat and helicopter data]')
axis_choice = input('Enter axes scaling choice: ');
if axis_choice == 0
    samp_time = 1;
    samp_freq = 1;
    xlabeltitle = 'Fraction of Sampling Frequency';
    ylabeltitle = 'Center Sample Number of Analysis Interval';
    f_spacing = input('Enter spacing in fractional frequency between frequency axis ticks: ');
    t_spacing = input('Enter spacing in samples between time axis ticks: ');
elseif axis_choice == 1
    samp_time = Ts;
    samp_freq = 1/Ts;
    xlabeltitle = 'Frequency (Hz)';
    ylabeltitle = 'Center Time of Analysis Interval (s)';
    f_spacing = input('Enter spacing in Hz between frequency axis ticks: ');
    t_spacing = input('Enter spacing in seconds between time axis ticks: ');
elseif axis_choice == 2
    samp_time = 1000*Ts;
    samp_freq = 1/(1000*Ts);
    xlabeltitle = 'Frequency (KHz)';
    ylabeltitle = 'Center Time of Analysis Interval (ms)';
    f_spacing = input('Enter spacing in KHz between frequency axis ticks: ');
    t_spacing = input('Enter spacing in milliseconds between time axis ticks: ');
end
step_time = samp_time*step;
step_freq = samp_freq/num_freqs;
if any(any(imag(data)~=0))
    data_type = 1;
    disp('Waveform is complex-valued. Real part left & imag part right of gram.')
    psd_save = psd;
    min_freq_index = 1;
else
    data_type = 0;
    disp('Waveform is real-valued. Plot left of gram.')
    min_freq_index = num_freqs/2 + 1;
    psd_save = psd(:,min_freq_index:num_freqs);
end
dB_reference =(1/max(max(abs(psd_save))));
dB_psd = 10*log10(dB_reference*abs(psd_save));
[rdb,cdb] = size(dB_psd);
dB_psd_temp = min(dB_psd,dB_top*ones(rdb,cdb));
dB_psd = max(dB_psd_temp,dB_bottom*ones(rdb,cdb));
% dB_psd = 10*log10( (1/max(max(abs(psd_save))))*abs(psd_save) + 10^(-dB_range/10) );
% Code prevents taking logarithm of 0 or - numbers; sets min and max
% dB plot levels
clear psd_save dB_psd_temp

fmax = samp_freq/2;                % max frequency in selected frequency units
tmax = num_times*step_time;        % max time in selected time units
t_axis = (1:num_times)*step_time;
f_tick = fix(fmax/f_spacing);
t_tick = fix(tmax/t_spacing);
YY = t_spacing*(0:t_tick)';
%YYY = (t_spacing/2)*(0:2*t_tick)';
if data_type == 0
    f_axis = (0:num_freqs/2-1)*step_freq;
    fmin = 0;
    XX = f_spacing*(0:f_tick)';
    %XXX = (f_spacing/2)*(0:2*f_tick)';
    axwidth = .835;
elseif data_type == 1
    f_axis = (-num_freqs/2:num_freqs/2-1)*step_freq;
    fmin = -fmax;
    XX = f_spacing*(-f_tick:f_tick)';
    %XXX = (f_spacing/2)*(-2*f_tick:2*f_tick)';
    axwidth = .705;
end
% Find nearest power of 2 greater than or equal to data length
fft_size = 2.^nextpow2(length(data));
tfr_long = periodogram_psd(fft_size,1,0,length(data),1,data);
if data_type == 0
   tfr_long = tfr_long(fft_size/2+1:fft_size);
end

fig1 = figure('PaperOrientation','landscape',...
              'PaperPosition',[1.1,1,9,6.5],...
              'Position',[10,10,820,620],...
              'Units','pixel');

%***************************************************************************************
% Figure title plot
ax_title   = axes('Position',[.16,.97,axwidth,.3],...
                  'Units','normalized',...
                  'Visible','off');
title_text = text('Position',[.5,0],...
                  'Units','normalized',...
                  'FontSize',14,...
                  'HorizontalAlignment','center',...
                  'VerticalAlignment','bottom',...
                  'String','COLOR  GRAM :  Localized Time  vs  Localized Frequency');

%***************************************************************************************
% Time-vs-frequency gram plot
axbottom = .15;
axheight = .79;
selection = 1;    % selection = input('Enter 0 for gray scale, 1 for color scale: ');
if selection == 0
    colormap(gray(256));
elseif selection == 1
    map = hsv(329);
    colormap(map(256:-1:1,:));
end
ax1 = axes('Position',[.16,axbottom,axwidth,axheight],...
           'Units','normalized',...
           'FontSize',9,...
           'Box','on',...
           'TickDir','out',...
           'NextPlot','add',...
           'XGrid','off',...
           'XTick',XX,...
           'XLim',[fmin,fmax],...
           'YTick',YY,...
           'YLim',[0,tmax]);
xlabel(xlabeltitle)
ylabel(ylabeltitle)
title(titletext,'FontSize',14)
imagesc(f_axis,t_axis,dB_psd)

%***************************************************************************************
%Real-signal-amplitude-vs-time plot to left of time-vs-frequency gram plot
t_height = ((tmax + interval*samp_time)/tmax)*axheight;
t_bottom = axbottom - (t_height - axheight)/2;

data_max = 1.05*max(max(abs(real(data))),max(abs(imag(data))));   %

ax2 = axes('Position',[.01,t_bottom,.11,t_height],...
           'Units','normalized',...
           'Box','on',...
           'NextPlot','add',...
           'FontSize',9,...
           'XLim',[-data_max,data_max],...
           'XDir','reverse',...
           'YTick',[],...
           'YTickLabelMode','manual',...
           'YTickLabel',[],...
           'YLim',[0,(length(data)-1)*samp_time]);
time_axis = (0:length(data)-1)*samp_time;
plot(real(data),time_axis,'k')
xlabel('Waveform')
if data_type == 1
    title('Real Part')
end
line([.6*data_max,.6*data_max],[0,interval*samp_time],'Color','r')
line([.65*data_max,.55*data_max],[0,0],'Color','r')
line([.65*data_max,.55*data_max],[interval*samp_time,interval*samp_time],'Color','r')

%***************************************************************************************
% Imag-signal-amplitude-vs-time plot to right of gram plot (if data is complex-valued)
if data_type == 1
    ax3 = axes('Position',[.88,t_bottom,.11,t_height],...
           'Units','normalized',...
           'Box','on',...
           'NextPlot','add',...
           'FontSize',9,...
           'XLim',[-data_max,data_max],...
           'XDir','reverse',...
           'YTick',[],...
           'YTickLabelMode','manual',...
           'YTickLabel',[],...
           'YLim',[0,(length(data)-1)*samp_time]);
    plot(imag(data),time_axis,'k')
    xlabel('Waveform')
    title('Imag Part')
end

%***************************************************************************************
% Full waveform PSD plot to bottom of gram plot
ax4 = axes('Position',[.16,.02,axwidth,.07],...
           'Units','normalized',...
           'Box','on',...
           'NextPlot','add',...
           'FontSize',9,...
           'TickDir','in',...
           'XTickLabelMode','manual',...
           'XTickLabel',[],...
           'XLim',[fmin,fmax],...
           'YLim',[-dB_range,2]);
if data_type == 0
   freq_axis = (0:fft_size/2-1)*(samp_freq/fft_size);
elseif data_type == 1
   freq_axis = (-fft_size/2:fft_size/2-1)*(samp_freq/fft_size);
end
dB_tfr_long = 10*log10( (1/max(max(tfr_long)))*abs(tfr_long) + 10^(-dB_range/10) );
plot(freq_axis,dB_tfr_long,'k')
xlabel('PSD of Entire Data Record')
ylabel('dB')

%***************************************************************************************
% Color bar legend to bottom left
ax5 = axes('Position',[.11,.015,.02,.10],...
           'Units','normalized',...
           'Box','on',...
           'NextPlot','add',...
           'FontSize',9,...
           'TickDir','out',...
           'XTickLabelMode','manual',...
           'XTickLabel',[],...
           'XLim',[1,2],...
           'YTick',[dB_bottom;dB_top],...
           'YLim',[dB_bottom,dB_top]);
vec = (dB_bottom:dB_top)';
barr = [vec vec];
imagesc((1:2),(dB_bottom:dB_top),barr)
ylabel('dB color code')
%